/* rtp_stream.c
 * RTP streams summary addition for ethereal
 *
 * $Id: $
 *
 * Copyright 2003, Alcatel Business Systems
 * By Lars Ruoff <lars.ruoff@alplog.fr>
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation,  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "rtp_stream.h"
#include "rtp_stream_dlg.h"

#include "globals.h"
#include "epan/packet_info.h"
#include "epan/epan_dissect.h"
#include "epan/filesystem.h"

#include "tap.h"
#include "register.h"
#include "packet-rtp.h"
#include "packet-udp.h"
#include "packet-ip.h"

#include "simple_dialog.h"

/*
#include <gtk/gtk.h>
#include "gtkglobals.h"
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fcntl.h>

#ifdef HAVE_IO_H
#include <io.h> /* open/close on win32 */
#endif

#ifndef O_BINARY
#define O_BINARY 0
#endif


/****************************************************************************/
/* the one and only global rtpstream_tapinfo_t structure */
static rtpstream_tapinfo_t the_tapinfo_struct =
	{0, NULL, 0, TAP_ANALYSE, NULL, NULL, 0, FALSE};


/****************************************************************************/
/* GCompareFunc style comparison function for _rtp_stream_info */
gint rtp_stream_info_cmp(gconstpointer aa, gconstpointer bb)
{
	const struct _rtp_stream_info* a = aa;
	const struct _rtp_stream_info* b = bb;

	if (a==b)
		return 0;
	if (a==NULL || b==NULL)
		return 1;
	if ((a->src_addr == b->src_addr)
		&& (a->src_port == b->src_port)
		&& (a->dest_addr == b->dest_addr)
		&& (a->dest_port == b->dest_port)
		&& (a->ssrc == b->ssrc)
		&& (a->pt == b->pt))
		return 0;
	else
		return 1;
}


/****************************************************************************/
/* when there is a [re]reading of packet's */
void rtpstream_reset(void *prs)
{
	rtpstream_tapinfo_t *tapinfo = prs;
	GList* list;

	if (tapinfo->mode == TAP_ANALYSE) {
		/* free the data items first */
		list = g_list_first(tapinfo->strinfo_list);
		while (list)
		{
			g_free(list->data);
			list = g_list_next(list);
		}
		g_list_free(tapinfo->strinfo_list);
		tapinfo->strinfo_list = NULL;
		tapinfo->nstreams = 0;
		tapinfo->npackets = 0;
	}

	++(tapinfo->launch_count);

	return;
}

/****************************************************************************/
/* here we can redraw the output */
/* not used yet */
void rtpstream_draw(void *prs _U_)
{
/*	gtk_signal_emit_by_name(top_level, "signal_rtpstream_update");*/
	rtpstream_dlg_update(the_tapinfo_struct.strinfo_list);
	return;
}


/****************************************************************************/
/* utility function for writing a sample to file in RAS format */
static
void ras_write_sample(rtp_sample_t* sample, FILE* file)
{
	rtp_sample_header_t net_order_header;

	net_order_header.rec_time = g_htonl(sample->header.rec_time);
	net_order_header.frame_length = g_htons(sample->header.frame_length);

	fwrite(&(net_order_header.rec_time), 1, 4, file);
	fwrite(&(net_order_header.frame_length), 1, 2, file);
	fwrite(sample->frame, 1, sample->header.frame_length, file);
}

/****************************************************************************/
/* whenever a RTP packet is seen by the tap listener */
int rtpstream_packet(void *prs, packet_info *pinfo, epan_dissect_t *edt _U_, void *vpri)
{
	rtpstream_tapinfo_t* tapinfo = prs;
	struct _rtp_info *rtpinfo = vpri;
	rtp_stream_info_t tmp_strinfo;
	rtp_stream_info_t *strinfo = NULL;
	guint8 *data;
	GList* list;

	rtp_sample_t sample;
	gchar sample_buf[400];

	/* we ignore packets that are not displayed */
/*
	if (pinfo->fd->flags.passed_dfilter == 0)
		return 0;
*/

	/* gather infos on the stream this packet is part of */
	g_memmove(&(tmp_strinfo.src_addr), pinfo->src.data, 4);
	tmp_strinfo.src_port = pinfo->srcport;
	g_memmove(&(tmp_strinfo.dest_addr), pinfo->dst.data, 4);
	tmp_strinfo.dest_port = pinfo->destport;
	tmp_strinfo.ssrc = rtpinfo->info_sync_src;
	tmp_strinfo.pt = rtpinfo->info_payload_type;
	tmp_strinfo.npackets = 0;
	tmp_strinfo.first_frame_num = pinfo->fd->num;
	tmp_strinfo.tag_vlan_error = 0;
	tmp_strinfo.tag_diffserv_error = 0;
	tmp_strinfo.vlan_id = 0;

	if (tapinfo->mode == TAP_ANALYSE) {
		/* check wether we already have a stream with these parameters in the list */
		list = g_list_first(tapinfo->strinfo_list);
		while (list)
		{
			if (rtp_stream_info_cmp(&tmp_strinfo, (rtp_stream_info_t*)(list->data))==0)
			{
				strinfo = (rtp_stream_info_t*)(list->data);  /*found!*/
				break;
			}
			list = g_list_next(list);
		}

		/* not in the list? then create a new entry */
		if (!strinfo) {
			strinfo = g_malloc(sizeof(rtp_stream_info_t));
			*strinfo = tmp_strinfo;  /* memberwise copy of struct */
			tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
		}

		/* increment the packets counter for this stream */
		++(strinfo->npackets);

		/* increment the packets counter of all streams */
		++(tapinfo->npackets);
	}
	else if (tapinfo->mode == TAP_SAVE) {
		if (rtp_stream_info_cmp(&tmp_strinfo, tapinfo->filter_stream_fwd)==0) {
			sample.header.rec_time = pinfo->fd->abs_secs*1000 + pinfo->fd->abs_usecs;
			sample.header.frame_length = rtpinfo->info_data_len;
			sample.frame = cfile.pd + pinfo->fd->pkt_len - rtpinfo->info_data_len;
			ras_write_sample(&sample, tapinfo->save_file);
		}
	}
	else if (tapinfo->mode == TAP_MARK) {
		if (rtp_stream_info_cmp(&tmp_strinfo, tapinfo->filter_stream_fwd)==0
			|| rtp_stream_info_cmp(&tmp_strinfo, tapinfo->filter_stream_rev)==0)
		{
			mark_frame(&cfile, pinfo->fd);
		}
	}

	return 0;
}

/****************************************************************************/
/* scan for RTP streams */
void rtpstream_scan()
{
	gboolean was_registered = the_tapinfo_struct.is_registered;
	if (!the_tapinfo_struct.is_registered)
		register_tap_listener_rtp_stream();

	the_tapinfo_struct.mode = TAP_ANALYSE;
	redissect_packets(&cfile);

	if (!was_registered)
		remove_tap_listener_rtp_stream();
}


/****************************************************************************/
/* save rtp dump of stream_fwd */
void rtpstream_save(rtp_stream_info_t* stream, const gchar *filename)
{
	gboolean was_registered = the_tapinfo_struct.is_registered;
	/* open file for saving */
	the_tapinfo_struct.save_file = fopen(filename, "wb");
	if (the_tapinfo_struct.save_file==NULL) {
		return;
	}

	if (!the_tapinfo_struct.is_registered)
		register_tap_listener_rtp_stream();

	the_tapinfo_struct.mode = TAP_SAVE;
	the_tapinfo_struct.filter_stream_fwd = stream;
	redissect_packets(&cfile);
	the_tapinfo_struct.mode = TAP_ANALYSE;

	if (!was_registered)
		remove_tap_listener_rtp_stream();

	/* XXX check for error at fclose? */
	fclose(the_tapinfo_struct.save_file);
}


/****************************************************************************/
/* mark packets in stream_fwd or stream_rev */
void rtpstream_mark(rtp_stream_info_t* stream_fwd, rtp_stream_info_t* stream_rev)
{
	gboolean was_registered = the_tapinfo_struct.is_registered;
	if (!the_tapinfo_struct.is_registered)
		register_tap_listener_rtp_stream();

	the_tapinfo_struct.mode = TAP_MARK;
	the_tapinfo_struct.filter_stream_fwd = stream_fwd;
	the_tapinfo_struct.filter_stream_rev = stream_rev;
	redissect_packets(&cfile);
	the_tapinfo_struct.mode = TAP_ANALYSE;

	if (!was_registered)
		remove_tap_listener_rtp_stream();
}


/****************************************************************************/
const rtpstream_tapinfo_t* rtpstream_get_info()
{
	return &the_tapinfo_struct;
}


/****************************************************************************/
/* TAP INTERFACE */
/****************************************************************************/

/****************************************************************************/
static void
rtpstream_init_tap(char *dummy _U_)
{
	/* XXX: never called? */
}


/* XXX just copied from gtk/rpc_stat.c */
void protect_thread_critical_region(void);
void unprotect_thread_critical_region(void);

/****************************************************************************/
void
remove_tap_listener_rtp_stream(void)
{
	if (the_tapinfo_struct.is_registered) {
		protect_thread_critical_region();
		remove_tap_listener(&the_tapinfo_struct);
		unprotect_thread_critical_region();

		the_tapinfo_struct.is_registered = FALSE;
	}
}


/****************************************************************************/
void
register_tap_listener_rtp_stream(void)
{
	gchar filter_text[256];
	GString *error_string;

	if (!the_tapinfo_struct.is_registered) {
		register_ethereal_tap("rtp", rtpstream_init_tap);

		sprintf(filter_text, "rtp && ip && !icmp");

		error_string = register_tap_listener("rtp", &the_tapinfo_struct,
			filter_text,
			rtpstream_reset, rtpstream_packet, rtpstream_draw);

		if (error_string != NULL) {
			simple_dialog(ESD_TYPE_WARN, NULL, error_string->str);
			g_string_free(error_string, TRUE);
			exit(1);
		}

		the_tapinfo_struct.is_registered = TRUE;
	}
}

